from zope.interface import implements, Interface
from OFS.SimpleItem import SimpleItem
from Products.CMFCore.WorkflowCore import WorkflowException
from plone.app.contentrules.browser.formhelper import AddForm, EditForm
from plone.contentrules.rule.interfaces import IExecutable, IRuleElementData
from zope.component import adapts
from zope.formlib import form
from zope import schema
from ubify.policy import CyninMessageFactory as _
import zope

class IQuarantineAction(Interface):
    """ QuarantineAction
    """
    ctypes = schema.Tuple(
        title=u"Content types to quarantine",
        value_type=schema.Choice(
            vocabulary="plone.app.vocabularies.ReallyUserFriendlyTypes"),
        default=("Discussion",),
        required=True,
        description=u"Which content types should "
                    "also be quarantined?")


class EnableQuarantineAction(SimpleItem):
    """ Enable/Disable Content Quarantine Action settings
    """
    implements(IQuarantineAction, IRuleElementData)

    ctypes = u''
    element = 'eea.cynadmin.actions.Quarantine'

    @property
    def summary(self):
        """ Summary
        """
        content_types = ("Discussion",)
        if self.ctypes:
            content_types = self.ctypes
        return "%s posted today by the offending user " \
               "will be quarantined" % (", ".join(content_types))




class EnableQuarantineActionExecutor(object):
    """ Quarantine Action executor
    """
    implements(IExecutable)
    adapts(Interface, IQuarantineAction, Interface)

    def __init__(self, context, element, event):
        self.context = context
        self.element = element
        self.event = event

    def __call__(self):
        obj = self.event.object
        wftool = obj.portal_workflow
        check_user_actity = user_activity(obj)
        if check_user_actity:
            ctypes = check_user_actity.get_user_posts_from_today(
                ctypes=self.element.ctypes)
            for brain in ctypes:
                obj = brain.getObject()
                if wftool.getInfoFor(obj, 'review_state') != "quarantine":
                    try:
                        wftool.doActionFor(obj, 'contentrule_quarantine')
                    except WorkflowException:
                        continue
        return True


def user_activity(obj):
    return obj.restrictedTraverse('check_user_activity', None)

def update_types_to_quarantine(context, data):
    check_user_activity = user_activity(context)
    if check_user_activity:
        site_props = check_user_activity.get_site_properties()
        value = data.values()[0]
        check_user_activity.set_portal_property(site_props,
                                        'typesToQuarantineContentFor', value)


class QuarantineAddForm(AddForm):
    """ Action edit form
    """
    form_fields = form.FormFields(IQuarantineAction)
    label = u"Enable quarantine action"
    description = u""
    form_name = u"Enable quarantine action"

    def create(self, data):
        action = EnableQuarantineAction()
        form.applyChanges(action, self.form_fields, data)
        update_types_to_quarantine(self.context, data)
        return action


class QuarantineEditForm(EditForm):
    """ Action edit form
    """
    form_fields = form.FormFields(IQuarantineAction)
    label = u"Edit quarantine action"
    description = u""
    form_name = u"Edit quarantine action"

    @form.action(_(u"label_save", default=u"Save"),
                 condition=form.haveInputWidgets,
                 name=u'save')
    def handle_save_action(self, action, data):
        update_types_to_quarantine(self.context, data)
        # original handle save action
        if form.applyChanges(self.context, self.form_fields, data, self.adapters):
            zope.event.notify(zope.lifecycleevent.ObjectModifiedEvent(self.context))
            self.status = "Changes saved"
        else:
            self.status = "No changes"

        nextURL = self.nextURL()
        if nextURL:
            self.request.response.redirect(self.nextURL())
        return ''
