import logging
from datetime import datetime

from zope.component import queryUtility
from ZODB.POSException import ConflictError

from plone.app.contentrules.actions.mail import IMailAction
from plone.contentrules.engine.interfaces import IRuleStorage
from plone.contentrules.engine.interfaces import IRuleAssignmentManager


from Products.CMFCore.interfaces import ISiteRoot
from Products.CMFCore.utils import getToolByName
from Products.CMFNotification.exceptions import MailHostNotFound
from Products.CMFNotification.utils import encodeMailHeaders
from Products.Five.browser import BrowserView
from Products.MaildropHost import MaildropHost

MAIL_HOST_META_TYPES = ('Mail Host', 'Secure Mail Host', 'Maildrop Host')
LOG = logging.getLogger('UnansweredDiscussions')

class UnansweredDiscussions(BrowserView):
    def __call__(self):
        pptool = getToolByName(self.context,'portal_properties')
        
        dayofweekformail = pptool.site_properties.getProperty('dayOfWeekForMail')
        if not dayofweekformail:
            dayofweekformail = 0
        
        if datetime.now().weekday() == dayofweekformail:
            self.sendMailForUnansweredDiscussions()
            
    def sendMailForUnansweredDiscussions(self):
        tosend = self.getNotCommentedDiscussions()
        
        mailhosts = self.context.superValues(MAIL_HOST_META_TYPES)
        if not mailhosts:
            raise MailHostNotFound
	
        bfound = False
        for mh in mailhosts:
            if isinstance(mh,MaildropHost):
                mailhost = mh
                bfound = True
            if bfound == True:
                break
	
        if bfound == False:
            mailhost = mailhosts[0]

        ptool = getToolByName(self, 'portal_properties').site_properties
        encoding = ptool.getProperty('default_charset', 'utf-8')
        portal = getToolByName(self,'portal_url').getPortalObject()
        message =   """From: %s <%s>
Subject: List of Unanswered/Pending Questions
Content-Type: text/html; charset="utf-8"
Content-Transfer-Encoding: 8bit
MIME-Version: 1.0

<h3>Scheduled list of unanswered/pending questions</h3>
The following questions/discussions have not been replied/commented yet:<br/><br/>""" % (portal.email_from_name, portal.email_from_address)

        message = message.encode(encoding)
        for address in tosend.keys():
            
            this_message = ('To: %s\n' % address) + message
            this_message += "<ul>"
            for discussion in tosend[address]:
                this_message += "<li>Created on ["+self.context.toLocalizedTime(discussion.creation_date, long_format=0)+"] &lt;<a href='"+discussion.absolute_url()+"/view'>"+discussion.Title()+"</a>&gt;</li>"
                
            this_message += "</ul>"
            this_message += "<br/><br/><hr/>This message is automatically generated from "+portal.title+", a web application powered by cyn.in.<br />Read more about cyn.in at <a href='http://cyn.in'>http://cyn.in</a>"
            this_message = this_message.encode(encoding)            
            
            try:
                mailhost.send(this_message)
            except ConflictError:
                raise
            except:
                LOG.error('Error while sendind '\
                          'mail: \n%s' % this_message,
                          exc_info=True)

        
    def getNotCommentedDiscussions(self):
        pptool = getToolByName(self.context,'portal_properties')
        cat = getToolByName(self.context,'portal_catalog')
        brains = cat({'nr_comments':0,'sort_on':'Date','sort_order':'reverse'})
        notcommented = [brain.getObject() for brain in brains]
            
        if notcommented:
            defaultaddressformail = pptool.site_properties.getProperty('addressForMail')
            if not defaultaddressformail:
                defaultaddressformail = ""
            
            ruleStorage = queryUtility(IRuleStorage)
            
            notcommenteddiscussions = []
            for obj in notcommented:
                notcommenteddiscussion = {}
                notcommenteddiscussion['discussion'] = obj
                parent = obj.aq_inner.aq_parent
                addresses = []
                while parent is not None:
                    shouldAdd = False
                    assignedRules = IRuleAssignmentManager(parent).values()
                    
                    for assignedRule in assignedRules:
                        if assignedRule.enabled and (parent == obj.aq_inner.aq_parent or assignedRule.bubbles):
                            rule = ruleStorage[assignedRule.__name__]
                            for ruleAction in rule.actions:
                                if IMailAction.providedBy(ruleAction):
                                    for address in ruleAction.recipients.split(','):
                                        if address not in addresses:
                                            addresses.append(address)
                        
                    if ISiteRoot.providedBy(parent):
                        parent = None
                    else:
                        parent = parent.aq_inner.aq_parent
                if not addresses:
                    addresses.append(defaultaddressformail)
                notcommenteddiscussion['addresses'] = addresses
                notcommenteddiscussions.append(notcommenteddiscussion)
                
            maildiscussions = {}
            for notcommenteddiscussion in notcommenteddiscussions:
                for address in notcommenteddiscussion['addresses']:
                    if address not in maildiscussions.keys():
                        maildiscussions[address] = []
                    maildiscussions[address].append(notcommenteddiscussion['discussion'])
                
            return maildiscussions
        return {}
