from DateTime.DateTime import DateTime
from Products.CMFCore.utils import getToolByName
from Products.Five.browser import BrowserView
from plone.memoize.view import memoize

class CheckUserActivity(BrowserView):

    def __init__(self, context, request):
        self.context = context
        self.request = request

    @memoize
    def get_user(self):
        mtool = self.context.portal_membership
        return mtool.getAuthenticatedMember()

    @memoize
    def get_properties(self):
        return getToolByName(self.context, 'portal_properties')

    def get_site_properties(self):
        return self.get_properties().site_properties

    def set_portal_property(self, property_manager, key, value,
                            value_type='String'):
        if property_manager.hasProperty(key):
            property_manager.manage_changeProperties(**{key:value})
        else:
            property_manager.manage_addProperty(key, value, value_type)

    @memoize
    def get_user_posts(self, types_to_searchfor_recent, user_id):
        catalog = getToolByName(self.context, 'portal_catalog')
        end = DateTime() + 0.1 # If we have some clock skew peek a little to the future
        start = DateTime() - 1
        # query on object added only Today
        date_range_query = { 'query':(start,end), 'range': 'min:max'}
        results = catalog({"created": date_range_query,
                        "sort_order":"reverse",
                        'Creator':user_id,
                        'portal_type': types_to_searchfor_recent,
                        'sort_on':'created'
                        })
        return results

    def get_user_posts_from_today(self, ctypes=None):
        user = self.get_user()
        user_id = user.id
        portal_types = 'Discussion'
        if ctypes:
            portal_types = ctypes
        content = self.get_user_posts(portal_types, user_id)
        today = DateTime().Day()
        matches = []
        for brain in content:
            if today == brain.created.Day():
                matches.append(brain)
        return matches

    @memoize
    def should_post(self):
        user = self.get_user()
        user_id = user.id

        if user.getGroups() != ['AuthenticatedUsers']:
            return True
        email = user.getProperty('email')
        # users with email from eea are allowed to post anytime
        if  email and 'eea.europa.eu' in email:
            return True

        # some users might exist with roles instead of group assignment
        roles = user.getRoles()
        if roles != ['Member', 'Authenticated']:
            return True

        site_props = self.get_site_properties()
        content_added_in_last_hours = site_props.getProperty(
            'contentAddedInLastHours') or 2
        maximum_content_allowed = site_props.getProperty(
            'contentAllowedInLastHours') or 4
        types_searched = site_props.getProperty('typesToQuarantineContentFor')\
                         or 'Discussion'
        types_to_searchfor_recent = types_searched
        results = self.get_user_posts(types_to_searchfor_recent, user_id)
        now = DateTime()
        today = now.day()
        count = 0
        # User shouldn't be able to post more than maximum allowed number
        # of content per checked timeframe

        for brain in results:
            brain_date = brain.created
            if today == brain_date.day():
                hours_passed = now.hour() - brain_date.hour()
                if hours_passed <= content_added_in_last_hours:
                    count += 1
        if count >= maximum_content_allowed:
            return False
        else:
            return True


    def can_post(self):
        return self.should_post()

