from ubify.cyninv2theme.portlets.openFlashChart import template
from ubify.cyninv2theme.portlets.openFlashChart_varieties import Pie, pie_value
from ubify.cyninv2theme.portlets.openFlashChart_varieties import HBar, x_axis_labels, hbar_value
from ubify.cyninv2theme.portlets.statistics import getchartsteps
from Products.CMFCore.utils import getToolByName
from eea.cynadmin.utils import eea_purposes

def makeBarChart(results):
    chart = template('')

    plot = HBar()
    objvalues = [k['count'] for k in results if k['count'] > 0]

    spaces = [j['id'] for j in results if j['count'] > 0]
    spaces.reverse()

    xlabels = x_axis_labels()
    xlabels.set_colour("#666666")
    chartsteps = 1.0
    if len(objvalues) > 0:
        chartsteps = getchartsteps(objvalues[-1],objvalues[0])
    chart.set_x_axis(offset = False, labels = xlabels,steps = chartsteps,colour="#cccccc",grid_colour="#f1f1f1")
    chart.set_y_axis(offset=True,labels = spaces,colour="#666666",grid_colour="#f1f1f1")

    for val in objvalues:
        plot.append_values(hbar_value((0, val), tooltip = '#right# discussions', colour = '#4092D8'))
    chart.add_element(plot)
    chart.set_tooltip(stroke=1,colour="#1f1f1f",bg_colour="#292929",title_style="font-size:12px;color:#ffffff;font-weight:bold",body_style="font-size:12px;color:#ffffff",behaviour="hover")

    chart.set_bg_colour("#FFFFFF")
    return chart.encode()

def makePieChart(results):
    chart = template('')

    plot = Pie(start_angle = 35, animate = True, values = [pie_value(val = k['count'],label = (k['id'],None, None)) for k in results],colours = ['#4092D8', '#57AC0B', '#CC0000', '#862DFF', '#FF6600',  '#00FFF6','#FF37D2', '#5251ff', '#F0EA80', '#abff00',], label_colour = '#666666')
    plot.set_tooltip('#label#: #val# of #total#<br>#percent# of 100%')
    plot.set_gradient_fill(True)
    plot.set_no_labels(False)

    chart.add_element(plot)
    chart.set_tooltip(stroke=1,colour="#1f1f1f",bg_colour="#292929",title_style="font-size:12px;color:#ffffff;font-weight:bold",body_style="font-size:12px;color:#ffffff",behaviour="hover")
    chart.set_bg_colour("#FFFFFF")
    return chart.encode()

def getDiscussionStats(context, strpath):
    cat = getToolByName(context, 'portal_catalog')
    brains = cat({'path':{'query':strpath, 'depth':2},'portal_type':'ContentSpace'})

    results = []
    for brain in brains:
        space = brain.getObject()
        strpath = "/".join(space.getPhysicalPath())
        result = {}
        result['id'] = space.title
        result['count'] = len(cat({'path':{'query':strpath, 'depth':1},'portal_type':'Discussion'}))
        results.append(result)
    results.sort(lambda x,y: cmp(x['count'],y['count']),reverse=True)
    return results

def getPurposesStats(context, strpath):
    cat = getToolByName(context, 'portal_catalog')

    results = []
    total = len(cat({'path':{'query':strpath}, 'portal_type':'Discussion'}))
    withPurpose = 0
    for purpose in eea_purposes:
        if purpose != 'Please select':
            result = {}
            result['id'] = purpose
            result['count'] = len(cat({'path':{'query':strpath},'purpose':purpose}))
            withPurpose += result['count']
            if result['count'] > 0:
                results.append(result)
    result = {}
    result['id'] = 'Uncategorized'
    result['count'] = total - withPurpose
    if result['count'] > 0:
        results.append(result)
    results.sort(lambda x,y: cmp(x['count'],y['count']),reverse=True)
    return results

def eeagetjsondata(context,records=10,type=None):
    site_encoding = context.plone_utils.getSiteEncoding()
    strpath = "/".join(context.getPhysicalPath())
    portal = context.portal_url.getPortalObject()

    try:
        from ubify.policy.config import contentroot_details
        rootid = contentroot_details['id']
        objRoot = getattr(portal,rootid)
        if context == objRoot:
            strpath = "/".join(portal.getPhysicalPath())
        else:
            strpath = "/".join(context.getPhysicalPath())
    except AttributeError:
        strpath = "/".join(context.getPhysicalPath())

    if type.lower() == "discussionstats_bar":
        results = getDiscussionStats(context, strpath)
        return makeBarChart(results)

    if type.lower() == "discussionstats_pie":
        results = getDiscussionStats(context, strpath)
        return makePieChart(results)

    if type.lower() == "purposestats_bar":
        results = getPurposesStats(context, strpath)
        return makeBarChart(results)

    if type.lower() == "purposestats_pie":
        results = getPurposesStats(context, strpath)
        return makePieChart(results)