from DateTime import DateTime
from zope.interface import implements
from zExceptions import Unauthorized
from zope.component import adapts
from archetypes.schemaextender.interfaces import ISchemaExtender
from archetypes.schemaextender.field import ExtensionField
from Products.Archetypes.atapi import StringField, SelectionWidget, \
                                    BooleanField, BooleanWidget
from ubify.coretypes.interfaces import IDiscussion
from Products.CMFCore.utils import getToolByName

eea_purposes = ['Please select', 'Administrative case work',
                'Commercial purpose (companies)',
                'Education (primary, secondary and tertiary)',
                'Employment/ Contract opportunity', 'General interest',
                'Information dissemination (incl. bookstores)',
                'Non-commercial application',
                ('Policy-making/ public participation (incl. international '
                'institutions and NGO)'),
                'Research (incl. university assignments)', 'N/A']

class _ExtensionStringField(ExtensionField, StringField): pass
class _ExtensionBooleanField(ExtensionField, BooleanField):
    def get(self, instance):
        wft = getToolByName(instance, 'portal_workflow')
        pptool = getToolByName(instance,'portal_properties')

        p_private_state = pptool.site_properties.getProperty('discussionPrivateState')
        if not p_private_state:
            p_private_state = 'private'

        state = wft.getInfoFor(instance, 'review_state')
        if state == p_private_state:
            return True
        else:
            return False

    def set(self, instance, value, **kwargs):
        if value is None:
            value = False

        wft = getToolByName(instance, 'portal_workflow') 
        pptool = getToolByName(instance,'portal_properties')
        p_workflow_id = pptool.site_properties.getProperty('discussionProfile')
        if not p_workflow_id:
            p_workflow_id = 'ubify_spaces_content_workflow'

        p_private_state = pptool.site_properties.getProperty('discussionPrivateState')
        if not p_private_state:
            p_private_state = 'private'

        p_retract_action = pptool.site_properties.getProperty('discussionRetractAction')
        if not p_retract_action:
            p_retract_action = 'retract'

        p_publish_action = pptool.site_properties.getProperty('discussionPublishAction')
        if not p_publish_action:
            p_publish_action = 'project-hide'

        workflows = wft.getWorkflowsFor(instance)

        workflow_id = ''
        for workflow in workflows:
            if workflow.id == p_workflow_id:
                workflow_id = workflow.id

        state = wft.getInfoFor(instance, 'review_state')
        state_value = False
        if state  == p_private_state:
            state_value = True
        if state_value != value:
            if workflow_id:
                if value:
                    try:
                        wft.doActionFor(instance, p_retract_action, workflow_id)
                    except Unauthorized:
                        # Discussions are already quarantined and as such we
                        # don't have the permission to change the state to retract
                        pass
                else:
                    wft.doActionFor(instance, p_publish_action, workflow_id)
#TODO: Check what state we set when the user selects to publish the discussion: published-anonymous, or published-contributors
#        BooleanField.set(self, instance, value, **kwargs)

class EEASchemaExtender(object):
    """ Extends schema with purpose field
    """
    implements(ISchemaExtender)
    adapts(IDiscussion)

    fields = (
        _ExtensionStringField(
            schemata='categorization',
            name='purpose',
            widget=SelectionWidget(
                label="Purpose",
                label_msgid='eea_label_purpuse',
                i18n_domain='eea.cynadmin',
            ),
            vocabulary = eea_purposes
        ),
        _ExtensionBooleanField(
            schemata='default',
            name='privatequestion',
            widget=BooleanWidget(
                label='Keep it private',
                label_msgid='eea_label_private',
                i18n_domain='eea.cynadmin',
            )
        )
    )

    def __init__(self, context):
        self.context = context

    def getFields(self):
        return self.fields


